@@ -14,7 +14,7 @@ from scipy.interpolate import interp1d
 ###############################################################################
 ## Spectral resampling utility
 ###############################################################################
-def ReSamplingMatrix(lorig , lresam):
+def ReSamplingMatrix(lorig , lresam, norm='True', extrap = False):
     '''
     Compute resampling matrix R_o2r, useful to convert a spectrum sampled at 
     wavelengths lorig to a new grid lresamp. Both are assumed to be uniform grids (ie, constant step).
@@ -55,11 +55,10 @@ def ReSamplingMatrix(lorig , lresam):
     >>> np.trapz(forig, lorig)
     5588.7178984824877
     '''
+    
     # Defs: steps & numbers
     dlorig  = lorig[1]  - lorig[0]
     dlresam = lresam[1] - lresam[0]
-#    Nlorig  = lorig.size
-#    Nlresam = lresam.size
     Nlorig  = len(lorig)
     Nlresam = len(lresam)
 
@@ -71,25 +70,38 @@ def ReSamplingMatrix(lorig , lresam):
 
     # Reset & fill resampling matrix
     ResampMat = np.zeros((Nlresam,Nlorig))
-    for i_r in range(Nlresam):
-        # inferior & superior lambdas representing the contibutio of each lorig bin to current lresam bin.
-        l_inf = np.where(lorig_low > lresam_low[i_r] , lorig_low , lresam_low[i_r])
-        l_sup = np.where(lorig_upp < lresam_upp[i_r] , lorig_upp , lresam_upp[i_r])
 
-        # lambda interval overlap of each lorig bin for current lresam bin. Negatives cliped to zero.
-        dl = (l_sup - l_inf).clip(0)
+    # Find out if the original array is simply a subset of the resampled.
+    # If it is, this routine already returns the correct resampling matrix
+    subset = lresam_subset_lorig(lresam, lorig, ResampMat)
 
-        # When a lresam bin is not fully (> 99%) covered by lorig pixels, then discard it (leave it with zeros).
-        # This will only happen at the edges of lorig.
-        if (0 < dl.sum() < 0.99* dlresam):
-            dl = 0 * lorig
+    if (not subset):
 
-        ResampMat[i_r,:] = dl
+        for i_r in range(Nlresam):
+            # inferior & superior lambdas representing the contribution of each lorig bin to current lresam bin.
+            l_inf = np.where(lorig_low > lresam_low[i_r] , lorig_low , lresam_low[i_r])
+            l_sup = np.where(lorig_upp < lresam_upp[i_r] , lorig_upp , lresam_upp[i_r])
+    
+            # lambda interval overlap of each lorig bin for current lresam bin. Negatives cliped to zero.
+            dl = (l_sup - l_inf).clip(0)
+    
+            # When a lresam bin is not fully (> 99%) covered by lorig pixels, then discard it (leave it with zeros).
+            # This will only happen at the edges of lorig.
+            if (0 < dl.sum() < 0.99* dlresam):
+                dl = 0 * lorig
+    
+            ResampMat[i_r,:] = dl
+    
+        if norm:
+            ResampMat = ResampMat / dlresam
 
+    # Fix extremes: extrapolate if needed
+    if (extrap):
+        extrap_ResampMat(ResampMat, lorig_low, lorig_upp, lresam_low, lresam_upp)
 
-    ResampMat = ResampMat / dlresam
     return ResampMat
 
+
 def ReSamplingMatrixNonUniform(lorig, lresam, extrap = False):
     '''
     Compute resampling matrix R_o2r, useful to convert a spectrum sampled at 
@@ -152,41 +164,47 @@ def ReSamplingMatrixNonUniform(lorig, lresam, extrap = False):
     lr_upp[-1] = lresam[-1] + (lresam[-1] - lresam[-2])/2
 
     
-    # Iterate over resampled lresam vector
-    for i_r in range(len(lresam)): 
-        
-        # Find in which bins lresam bin within lorig bin
-        bins_resam = np.where( (lr_low[i_r] < lo_upp) & (lr_upp[i_r] > lo_low) )[0]
+    # Find out if the original array is simply a subset of the resampled.
+    # If it is, this routine already returns the correct resampling matrix
+    subset = lresam_subset_lorig(lresam, lorig, matrix)
 
-        # On these bins, eval fraction of resamled bin is within original bin.
-        for i_o in bins_resam:
+    if (not subset):
+
+        # Iterate over resampled lresam vector
+        for i_r in range(len(lresam)): 
             
-            aux = 0
-            
-            d_lr = lr_upp[i_r] - lr_low[i_r]
-            d_lo = lo_upp[i_o] - lo_low[i_o]
-            d_ir = lo_upp[i_o] - lr_low[i_r]  # common section on the right
-            d_il = lr_upp[i_r] - lo_low[i_o]  # common section on the left
-            
-            # Case 1: resampling window is smaller than or equal to the original window.
-            # This is where the bug was: if an original bin is all inside the resampled bin, then
-            # all flux should go into it, not then d_lr/d_lo fraction. --Natalia@IoA - 21/12/2012
-            if (lr_low[i_r] > lo_low[i_o]) & (lr_upp[i_r] < lo_upp[i_o]):
-                aux += 1.
+            # Find in which bins lresam bin within lorig bin
+            bins_resam = np.where( (lr_low[i_r] < lo_upp) & (lr_upp[i_r] > lo_low) )[0]
+    
+            # On these bins, eval fraction of resamled bin is within original bin.
+            for i_o in bins_resam:
                 
-            # Case 2: resampling window is larger than the original window.
-            if (lr_low[i_r] < lo_low[i_o]) & (lr_upp[i_r] > lo_upp[i_o]):
-                aux += d_lo / d_lr
-
-            # Case 3: resampling window is on the right of the original window.
-            if (lr_low[i_r] > lo_low[i_o]) & (lr_upp[i_r] > lo_upp[i_o]):
-                aux += d_ir / d_lr
-
-            # Case 4: resampling window is on the left of the original window.
-            if (lr_low[i_r] < lo_low[i_o]) & (lr_upp[i_r] < lo_upp[i_o]):
-                aux += d_il / d_lr
-
-            matrix[i_r, i_o] += aux
+                aux = 0
+                
+                d_lr = lr_upp[i_r] - lr_low[i_r]
+                d_lo = lo_upp[i_o] - lo_low[i_o]
+                d_ir = lo_upp[i_o] - lr_low[i_r]  # common section on the right
+                d_il = lr_upp[i_r] - lo_low[i_o]  # common section on the left
+                
+                # Case 1: resampling window is smaller than or equal to the original window.
+                # This is where the bug was: if an original bin is all inside the resampled bin, then
+                # all flux should go into it, not then d_lr/d_lo fraction. --Natalia@IoA - 21/12/2012
+                if (lr_low[i_r] >= lo_low[i_o]) & (lr_upp[i_r] <= lo_upp[i_o]):
+                    aux += 1.
+                    
+                # Case 2: resampling window is larger than the original window.
+                if (lr_low[i_r] < lo_low[i_o]) & (lr_upp[i_r] > lo_upp[i_o]):
+                    aux += d_lo / d_lr
+    
+                # Case 3: resampling window is on the right of the original window.
+                if (lr_low[i_r] > lo_low[i_o]) & (lr_upp[i_r] > lo_upp[i_o]):
+                    aux += d_ir / d_lr
+    
+                # Case 4: resampling window is on the left of the original window.
+                if (lr_low[i_r] < lo_low[i_o]) & (lr_upp[i_r] < lo_upp[i_o]):
+                    aux += d_il / d_lr
+    
+                matrix[i_r, i_o] += aux
 
 
     # Fix matrix to be exactly = 1 ==> TO THINK
@@ -195,21 +213,179 @@ def ReSamplingMatrixNonUniform(lorig, lresam, extrap = False):
     
     # Fix extremes: extrapolate if needed
     if (extrap):
-        
-        bins_extrapl = np.where( (lr_low < lo_low[0])  )[0]
-        bins_extrapr = np.where( (lr_upp > lo_upp[-1]) )[0]
-
-        if (len(bins_extrapl) > 0) & (len(bins_extrapr) > 0):
-            io_extrapl = np.where( (lo_low >= lr_low[bins_extrapl[0]])  )[0][0]
-            io_extrapr = np.where( (lo_upp <= lr_upp[bins_extrapr[0]])  )[0][-1]
-
-            matrix[bins_extrapl, io_extrapl] = 1.
-            matrix[bins_extrapr, io_extrapr] = 1.
-
+        extrap_ResampMat(matrix, lo_low, lo_upp, lr_low, lr_upp)
         
     return matrix
 
 
+def ReSamplingMatrixNonUniform_alt(lorig , lresam, norm='True', extrap = False):
+    '''
+    This is the pythonic equivalent to ReSamplingMatrixNonUniform.
+    However, because the resampling matrix is so sparse, this version
+    is *not* as fast as ReSamplingMatrixNonUniform when tested
+    against arrays of sizes ~3000. It might be faster in some cases,
+    so it remains here. It *is* faster than ReSamplingMatrix.
+
+    ----
+    
+    Compute resampling matrix R_o2r, useful to convert a spectrum sampled at 
+    wavelengths lorig to a new grid lresamp. Both are assumed to be uniform grids (ie, constant step).
+    Input arrays lorig and lresamp are the bin centres of the original and final lambda-grids.
+    ResampMat is a Nlresamp x Nlorig matrix, which applied to a vector F_o (with Nlorig entries) returns
+    a Nlresamp elements long vector F_r (the resampled spectrum):
+
+        [[ResampMat]] [F_o] = [F_r]
+
+    Based on (but less general than) STARLIGHT's routine RebinSpec. Conserves flux, except for 
+    possible loss at the blue & red edges of lorig (1st and last pixels).
+
+    ElCid@Sanchica - 09/Feb/2012
+    
+    
+    Parameters
+    ----------
+    lorig : array_like
+            Original spectrum lambda array.
+    
+    lresam : array_like
+             Spectrum lambda array in which the spectrum should be sampled.        
+        
+    Returns
+    -------
+    ResampMat : array_like
+                Resample matrix. 
+    
+    Examples
+    --------
+    >>> lorig = np.linspace(3400, 8900, 4623)
+    >>> lresam = np.linspace(3400, 8900, 9000)
+    >>> forig = np.random.normal(size=len(lorig))**2
+    >>> matrix = slut.ReSamplingMatrix(lorig, lresam)
+    >>> fresam = np.dot(matrix, forig)
+    >>> np.trapz(fresam, lresam)
+    5588.7178984840939
+    >>> np.trapz(forig, lorig)
+    5588.7178984824877
+    '''
+
+    # Init ResampMatrix
+    ResampMat__ro = np.zeros((len(lresam), len(lorig)))
+    
+    # Define lambda ranges (low, upp) for original and resampled.
+    lo_low = np.zeros(len(lorig))
+    lo_low[1:] = (lorig[1:] + lorig[:-1])/2
+    lo_low[0] = lorig[0] - (lorig[1] - lorig[0])/2 
+
+    lo_upp = np.zeros(len(lorig))
+    lo_upp[:-1] = lo_low[1:]
+    lo_upp[-1] = lorig[-1] + (lorig[-1] - lorig[-2])/2
+
+    lr_low = np.zeros(len(lresam))
+    lr_low[1:] = (lresam[1:] + lresam[:-1])/2
+    lr_low[0] = lresam[0] - (lresam[1] - lresam[0])/2
+    
+    lr_upp = np.zeros(len(lresam))
+    lr_upp[:-1] = lr_low[1:]
+    lr_upp[-1] = lresam[-1] + (lresam[-1] - lresam[-2])/2
+
+    
+    # Find out if the original array is simply a subset of the resampled.
+    # If it is, this routine already returns the correct resampling matrix
+    subset = lresam_subset_lorig(lresam, lorig, ResampMat__ro)
+
+    if (not subset):
+        
+        # Create comparison matrixes for lower and upper bin limits
+        na = np.newaxis
+        lo_low__ro = lo_low[na, ...]
+        lo_upp__ro = lo_upp[na, ...]
+        lr_low__ro = lr_low[..., na]
+        lr_upp__ro = lr_upp[..., na]
+    
+        # Find in which bins lresam bin within lorig bin
+        ff = (lr_low__ro < lo_upp__ro) & (lr_upp__ro > lo_low__ro)
+    
+        # Eval fraction of resamled bin is within original bin
+        d_lr = lr_upp__ro - lr_low__ro
+        d_lo = lo_upp__ro - lo_low__ro
+        d_ir = lo_upp__ro - lr_low__ro  # common section on the right
+        d_il = lr_upp__ro - lo_low__ro  # common section on the left
+    
+        # Case 1: resampling window is smaller than or equal to the original window.
+        f1 = (ff) & (lr_low__ro >= lo_low__ro) & (lr_upp__ro <= lo_upp__ro)
+        if (f1.sum() > 0):
+            ResampMat__ro[f1] += 1.
+    
+        # Case 2: resampling window is larger than the original window.
+        f2 = (ff) & (lr_low__ro < lo_low__ro) & (lr_upp__ro > lo_upp__ro)
+        if (f2.sum() > 0):
+            ResampMat__ro[f2] += (d_lo / d_lr)[f2]
+    
+        # Case 3: resampling window is on the right of the original window.
+        f3 = (ff) & (lr_low__ro > lo_low__ro) & (lr_upp__ro > lo_upp__ro)
+        if (f3.sum() > 0):
+            ResampMat__ro[f3] += (d_ir / d_lr)[f3]
+    
+        # Case 4: resampling window is on the left of the original window.
+        f4 = (ff) & (lr_low__ro < lo_low__ro) & (lr_upp__ro < lo_upp__ro)
+        if (f4.sum() > 0):
+            ResampMat__ro[f4] += (d_il / d_lr)[f4]
+
+    # Fix extremes: extrapolate if needed
+    if (extrap):
+        extrap_ResampMat(ResampMat__ro, lo_low, lo_upp, lr_low, lr_upp)
+
+    return ResampMat__ro
+
+
+def lresam_subset_lorig(lresam, lorig, ResampMat__ro):
+    '''
+    Finds out if lorig is a subset of lresam. If so,
+    create a `diagonal' resampling matrix (in place).
+    '''
+
+    subset = False
+    tol = 1.e-5
+    
+    ff = (lresam >= lorig[0]) & (lresam <= lorig[-1])
+    lresam_sub = lresam[ff]
+
+    N_lo = len(lorig)
+
+    if (len(lresam_sub) == N_lo):
+        Nok = (abs(lorig - lresam_sub) < tol).sum()
+
+        if (Nok == N_lo):
+            subset = True
+            ilow = np.where(ff)[0][0]
+            iupp = np.where(ff)[0][-1]
+    
+            io_diag = np.arange(len(lorig))
+            ir_diag = np.arange(ilow, iupp+1)
+            ResampMat__ro[ir_diag, io_diag] = 1.
+
+    return subset
+
+
+def extrap_ResampMat(ResampMat__ro, lo_low, lo_upp, lr_low, lr_upp):
+    '''
+    Extrapolate resampling matrix on the borders, i.e.,
+    by making it copy the first and the last bins.
+
+    This modifies ResampMat__ro in place.
+    '''
+    
+    bins_extrapl = np.where( (lr_low < lo_low[0])  )[0]
+    bins_extrapr = np.where( (lr_upp > lo_upp[-1]) )[0]
+
+    if (len(bins_extrapl) > 0) & (len(bins_extrapr) > 0):
+        io_extrapl = np.where( (lo_low >= lr_low[bins_extrapl[0]]) )[0][0]
+        io_extrapr = np.where( (lo_upp <= lr_upp[bins_extrapr[0]]) )[0][-1]
+
+        ResampMat__ro[bins_extrapl, io_extrapl] = 1.
+        ResampMat__ro[bins_extrapr, io_extrapr] = 1.
+
+    
 ###############################################################################
 ## Starlight base utility
 ###############################################################################
@@ -455,8 +631,7 @@ def light2MassIni(popx, fbase_norm, Lobs_norm, q_norm, A_V):
     Lobs *= Lobs_norm
     Lobs *= 10.0**(0.4 * q_norm * A_V)
     
-    ndim = len(Lobs.shape)
-    Mini = Lobs / arrayAsRowMatrix(fbase_norm, ndim)
+    Mini = Lobs / arrayAsRowMatrix(fbase_norm, Lobs.ndim)
     return Mini
 
 
@@ -570,8 +745,8 @@ def calcPopmu(popx, fbase_norm, Lobs_norm, Mstars, q_norm, A_V):
     ndim = len(Mini.shape)
     Mcor = Mini * arrayAsRowMatrix(Mstars, ndim)
     
-    popmu_ini = Mini / Mini.sum(axis=-1) * 100.0
-    popmu_cor = Mcor / Mcor.sum(axis=-1) * 100.0
+    popmu_ini = Mini / Mini.sum(axis=1).sum(axis=0) * 100.0
+    popmu_cor = Mcor / Mcor.sum(axis=1).sum(axis=0) * 100.0
     return popmu_ini, popmu_cor 
 
 
@@ -594,10 +769,93 @@ def arrayAsRowMatrix(a, ndim):
     a_reshaped : array
         The same array as ``a``, reshaped to ``ndim`` dimensions.
     '''
-    old_ndim = len(a.shape)
-    if len(a.shape) >= ndim:
+    if a.ndim == ndim:
+        return a
+    if a.ndim > ndim:
         raise ValueError('number of dimensions of input must be smaller than %d.' % ndim)
     new_shape = np.ones(ndim, dtype=int)
-    new_shape[:old_ndim] = a.shape
+    new_shape[:a.ndim] = a.shape
     return a.reshape(new_shape)
 
+
+################################################################################
+
+def gaussVelocitySmooth(lo, fo, v0, sig, ls=None, n_u=31, n_sig=6):
+    '''
+    c # Smooth an input spectrum fo(lo) with a Gaussian centered at velocity
+    c # v = v0 and with sig as velocity dispersion. The output is fs(ls).
+    c # The convolution is done with a BRUTE FORCE integration of n_u points
+    c # from u = -n_sig to u = +n_sig, where u = (v - v0) / sig.
+    c # OBS: Assumes original spectrum fo(lo) is EQUALLY SPACED in lambda!!
+    C #      Units of v0 & sig = km/s
+    c # Cid@Lynx - 05/Dec/2002
+    
+    c # Added n_u as a parameter!
+    c # Cid@Lynx - 16/Mar/2003
+    
+    *ETC* !AKI! gfortran does not like do u=u_low,u_upp,du! But works! Fix
+    *     after other tests are done! Cid@Lagoa - 11/Jan/2011
+    
+    *     This version of GaussSmooth replaces the u-loop by one in an
+    *     integer index i_u. The differences wrt are minute (< 1e-5) and due
+    *     to precision...
+    *
+    *      ElCid@Sanchica - 16/Ago/2011
+    
+    *ATT* Promoted to new oficial GaussSmooth without much testing!
+    *      Previous version removed to skip compilation warnings...
+    *    ElCid@Sanchica - 08/Oct/2011
+    
+    *ATT* Found an array bounds-bug when compiling with gfortran ... -fbounds-check
+    *      Ex: Fortran runtime error: Index '3402' of dimension 1 of array 'fo' above upper bound of 3401
+    *      Happens only when sig is very large. The old routine is kept below, and documents the bug briefly.
+    *    ElCid@Sanchica - 05/Nov/2011
+    '''
+    c  = 2.997925e5
+
+    if n_u < 5:
+        raise ValueError('n_u=%d too small to integrate properly.' % n_u)
+
+    if ls is None:
+        ls = lo
+        
+# Parameters for the brute-force integration in velocity
+    u = np.linspace(-n_sig, n_sig, n_u)
+    du = u[1] - u[0]
+    d_lo = lo[1] - lo[0]
+
+    Ns = len(ls)
+    No = len(lo)
+    fs = np.empty(Ns, dtype=fo.dtype)
+
+# loop over ls, convolving fo(lo) with a gaussian
+    for i_s in xrange(Ns):
+
+# reset integral of {fo[ll = ls/(1+v/c)] * Gaussian} & start integration
+        sum_fg = 0.
+        for _u in u:
+
+# define velocity & lambda corresponding to u
+            v  = v0 + sig * _u
+            ll = ls[i_s] / (1.0 + v/c)
+
+# find fo flux for lambda = ll
+            ind = int((ll - lo[0]) / d_lo)
+            if ind < 0: 
+                ff = fo[0]
+            elif ind >= No-1:
+                ff = fo[No-1]
+            else:
+                a  = (fo[ind+1] - fo[ind]) / (lo[ind+1] - lo[ind])
+                b  = fo[ind] - a * lo[ind]
+                ff = a * ll + b
+
+# smoothed spectrum
+            sum_fg = sum_fg + ff * np.exp(-(_u**2/2.)) 
+        fs[i_s] = sum_fg * du / np.sqrt(2. * np.pi)
+    return fs
+
+
+################################################################################\
+
+
